/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "cubo.h"

Cubo::Cubo( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer ), parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	crea_cubo( c1, 4.0 );
	// creaCubo( c2, 8.0 );
}


void Cubo::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Cubo::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Cubo::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Cubo::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	// En z estn en coordenadas negativas
	if( w <= h )
		glOrtho ( -10.0, 10.0, -10.0/aspect, 10.0/aspect, -10.0, 10.0);
	else
		glOrtho (-10.0*aspect, 10.0*aspect, -10.0, 10.0,  -10.0, 10.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	emit changeSize ( );
}

QSizePolicy Cubo::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Cubo::crea_cubo( float v[][3], float l )
{
	v[0][0] = l; v[0][1] = 0; v[0][2] = 0;
	v[1][0] = l; v[1][1] = 0; v[1][2] = l;
	v[2][0] = 0; v[2][1] = 0; v[2][2] = l;
	v[3][0] = 0; v[3][1] = 0; v[3][2] = 0;

	v[4][0] = l; v[4][1] = l; v[4][2] = 0;
	v[5][0] = l; v[5][1] = l; v[5][2] = l;
	v[6][0] = 0; v[6][1] = l; v[6][2] = l;
	v[7][0] = 0; v[7][1] = l; v[7][2] = 0;
}

void Cubo::dibuja_cubo( float v[][3] )
{
	int caras[6][4] = { {5,6,2,1}, {1,2,3,0}, {0,3,7,4}, {4,7,6,5},
	                {0,4,5,1}, {2,6,7,3} };

	float color[6][3] = { {1.0,1.0,1.0}, {1.0,1.0,0.0}, {0.0,0.0,1.0},
	                      {1.0,0.0,1.0}, {1.0,0.0,0.0}, {0.0,1.0,0.0} };
	int i;

	for( i=0; i<6; i++ ){
		glBegin( GL_QUADS );
		glVertex3fv( &v[ caras[i][0] ][0] );
		glVertex3fv( &v[ caras[i][1] ][0] );
		glVertex3fv( &v[ caras[i][2] ][0] );
		glColor3fv( &color[i][0] );
		glVertex3fv( &v[ caras[i][3] ][0] );
		glEnd( );
	}
}

void Cubo::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix( );
	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);

	dibuja_cubo( c1 );

	glPopMatrix( );

	swapBuffers( );
}

void Cubo::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);

	glShadeModel( GL_FLAT );
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */
}


